@@ -13,6 +13,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#ifndef __STDC_FORMAT_MACROS
+#define __STDC_FORMAT_MACROS  // before inttypes.h
+#endif
+#include <inttypes.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>  // memcpy
@@ -218,9 +222,6 @@ HWY_NOINLINE void TestAllBaseCase() {
 #if defined(_MSC_VER)
   return;
 #endif
-  // Only enable EMU128 on x86 - it's slow on emulators.
-  if (!HWY_ARCH_X86 && (HWY_TARGET == HWY_EMU128)) return;
-
   TestBaseCase<TraitsLane<OrderAscending<int32_t> > >();
   TestBaseCase<TraitsLane<OrderDescending<int64_t> > >();
   TestBaseCase<Traits128<OrderAscending128> >();
@@ -356,9 +357,6 @@ static HWY_NOINLINE void TestPartition() {
 }
 
 HWY_NOINLINE void TestAllPartition() {
-  // Only enable EMU128 on x86 - it's slow on emulators.
-  if (!HWY_ARCH_X86 && (HWY_TARGET == HWY_EMU128)) return;
-
   TestPartition<TraitsLane<OrderAscending<int16_t> > >();
   TestPartition<TraitsLane<OrderDescending<int32_t> > >();
   TestPartition<TraitsLane<OrderAscending<int64_t> > >();
@@ -490,9 +488,6 @@ void TestSort(size_t num_lanes) {
 #if defined(_MSC_VER)
   return;
 #endif
-  // Only enable EMU128 on x86 - it's slow on emulators.
-  if (!HWY_ARCH_X86 && (HWY_TARGET == HWY_EMU128)) return;
-
   using Order = typename Traits::Order;
   using LaneType = typename Traits::LaneType;
   using KeyType = typename Traits::KeyType;
